@@ -5,19 +5,30 @@ function (logPost, current.shapes, step, which, iters = 6L) {
     tt <- runif(1L, 0, step)
     L <- current.shape - tt
     R <- current.shape + step - tt
-    while (L > 0 && logPost(L, which)[[1L]] > logy) {
+    count <- 0L
+    lP.val <- logPost(L, which)[[1L]]
+    while (L > 0 && !is.na(lP.val) && lP.val > logy) {
         L <- L - step
+        count <- count + 1L
+        if (count > iters)
+            break
     }
-    while (logPost(R, which)[[1L]] > logy) {
+    count <- 0L
+    lP.val <- logPost(R, which)[[1L]]
+    while (!is.na(lP.val) && lP.val > logy) {
         R <- R + step
+        count <- count + 1L
+        if (count > iters)
+            break
     }
-    L <- max(0, L)
+    L <- max(0, L, na.rm = TRUE)
     count <- 0L
     repeat {
         count <- count + 1L
         new.shape <- runif(1L, L, R)
         new.logPost <- logPost(new.shape, which)
-        if (new.logPost[[1L]] >= logy || count > iters)
+        new.logPost.val <- new.logPost[[1L]]
+        if (is.na(new.logPost.val) || new.logPost.val >= logy || count > iters)
             break
         if (new.shape < current.shape) {
             L <- new.shape
@@ -25,5 +36,6 @@ function (logPost, current.shapes, step, which, iters = 6L) {
             R <- new.shape
         }
     }
-    c(list(new.shape = new.shape, fail = count > iters), new.logPost)
+    c(list(new.shape = new.shape, fail = count > iters || is.na(new.logPost.val)), 
+      new.logPost)
 }
